@@ -28,7 +28,21 @@ _trunk_version =  (float('inf'), float('inf'), float('inf'), float('inf'))
 # that means the solver has been fixed and that particular case should
 # no longer exist in the list of expected failures
 ExpectedFailures = {}
+#                                                                                                                                              
+# MOSEK                                                                                                                                        
+#                                                                                                                                              
 
+ExpectedFailures['mosek', 'python', 'QCP_simple'] = \
+    (lambda v: True,
+     "Conic constraints not yet handled by this interface")
+
+ExpectedFailures['mosek', 'python', 'QCP_simple_nosuffixes'] = \
+    (lambda v: True,
+     "Conic constraints not yet handled by this interface")
+
+ExpectedFailures['mosek', 'python', 'MIQCP_simple'] = \
+    (lambda v: True,
+     "Conic constraints not yet handled by this interface")
 #
 # CPLEX
 #
@@ -78,7 +92,7 @@ ExpectedFailures['glpk', 'mps', 'LP_duals_maximize'] = \
 #
 
 ExpectedFailures['cbc', 'lp', 'LP_duals_maximize'] = \
-    (lambda v: v <= _trunk_version,
+    (lambda v: v <= (2,10,1,0),
     "For a maximization problem where a variable is pushed to its "
     "lower bound, Cbc reports the reduced cost as a positive number. In "
     "practice this should be reported as a negative number. A ticket has "
