@@ -31,24 +31,33 @@ import pyomo.common
 print("import: 'pyomo.contrib'")
 import pyomo.contrib
 
+print("import: 'pyomo.contrib.derivatives'")
+import pyomo.contrib.derivatives
+
 print("import: 'pyomo.contrib.example'")
 import pyomo.contrib.example
 
 print("import: 'pyomo.contrib.example.plugins'")
 import pyomo.contrib.example.plugins
 
+print("import: 'pyomo.contrib.fbbt'")
+import pyomo.contrib.fbbt
+
+print("import: 'pyomo.contrib.gdpbb'")
+import pyomo.contrib.gdpbb
+
 print("import: 'pyomo.contrib.gdp_bounds'")
 import pyomo.contrib.gdp_bounds
 
-print("import: 'pyomo.contrib.gdp_bounds.plugins'")
-import pyomo.contrib.gdp_bounds.plugins
-
 print("import: 'pyomo.contrib.gdpopt'")
 import pyomo.contrib.gdpopt
 
 print("import: 'pyomo.contrib.mcpp'")
 import pyomo.contrib.mcpp
 
+print("import: 'pyomo.contrib.mindtpy'")
+import pyomo.contrib.mindtpy
+
 print("import: 'pyomo.contrib.multistart'")
 import pyomo.contrib.multistart
 
@@ -58,8 +67,8 @@ import pyomo.contrib.parmest
 print("import: 'pyomo.contrib.parmest.examples'")
 import pyomo.contrib.parmest.examples
 
-print("import: 'pyomo.contrib.parmest.examples.mea'")
-import pyomo.contrib.parmest.examples.mea
+print("import: 'pyomo.contrib.parmest.examples.reactor_design'")
+import pyomo.contrib.parmest.examples.reactor_design
 
 print("import: 'pyomo.contrib.parmest.examples.rooney_biegler'")
 import pyomo.contrib.parmest.examples.rooney_biegler
@@ -73,6 +82,30 @@ import pyomo.contrib.preprocessing
 print("import: 'pyomo.contrib.preprocessing.plugins'")
 import pyomo.contrib.preprocessing.plugins
 
+print("import: 'pyomo.contrib.pynumero'")
+import pyomo.contrib.pynumero
+
+print("import: 'pyomo.contrib.pynumero.algorithms'")
+import pyomo.contrib.pynumero.algorithms
+
+print("import: 'pyomo.contrib.pynumero.examples'")
+import pyomo.contrib.pynumero.examples
+
+print("import: 'pyomo.contrib.pynumero.extensions'")
+import pyomo.contrib.pynumero.extensions
+
+print("import: 'pyomo.contrib.pynumero.interfaces'")
+import pyomo.contrib.pynumero.interfaces
+
+print("import: 'pyomo.contrib.pynumero.linalg'")
+import pyomo.contrib.pynumero.linalg
+
+print("import: 'pyomo.contrib.pynumero.sparse'")
+import pyomo.contrib.pynumero.sparse
+
+print("import: 'pyomo.contrib.satsolver'")
+import pyomo.contrib.satsolver
+
 print("import: 'pyomo.contrib.sensitivity_toolbox'")
 import pyomo.contrib.sensitivity_toolbox
 
@@ -82,6 +115,9 @@ import pyomo.contrib.sensitivity_toolbox.examples
 print("import: 'pyomo.contrib.trustregion'")
 import pyomo.contrib.trustregion
 
+print("import: 'pyomo.contrib.viewer'")
+import pyomo.contrib.viewer
+
 print("import: 'pyomo.core'")
 import pyomo.core
 
